uses
  crt;

const
  pawn_value = 1;
  knight_value = 3;
  bishop_value = 3;
  rook_value = 5;
  queen_value = 9;

type
  board = array [1..8, 1..8] of char;
  colour = (black, white);
  move_type =
    (none, normal, capture, promote, castle_king_side, castle_queen_side);
  chess_move_p = ^chess_move;
  chess_move = record
		 prev                : chess_move_p;
		 b                   : board;
		 c                   : colour;
		 from_col, from_row,
		 to_col, to_row      : byte;
		 value               : real;
		 piece		     : char;
		 case move : move_type of
		   capture : ( take_piece    : char );
		   promote : ( promote_piece : char );
	       end;

  game = record
	   move                  : chess_move_p;
	   w_king_moved,
	   w_kings_rook_moved,
	   w_queens_rook_moved,
	   b_king_moved,
	   b_kings_rook_moved,
	   b_queens_rook_moved   : boolean;
	 end;

var
  b : board;

const initial_board : board  = (
  'rnbkqbnr',
  'pppppppp',
  '        ',
  '        ',
  '        ',
  '        ',
  'PPPPPPPP',
  'RNBKQBNR'
);

const initial_move : chess_move = (
  prev : nil;
  b : ( 'rnbkqbnr',
	'pppppppp',
	'        ',
	'        ',
	'        ',
	'        ',
	'PPPPPPPP',
	'RNBKQBNR' );
  c : white;
  from_col : 0; from_row : 0;
  to_col : 0; to_row : 0;
  value : 8 * pawn_value + 2 * knight_value +
	  2 * bishop_value + 2 * rook_value + queen_value;
  piece : ' ';
  move : none
);

const initial_position : game = (
	   move                : nil;
	   w_king_moved        : false;
	   w_kings_rook_moved  : false;
	   w_queens_rook_moved : false;
	   b_king_moved        : false;
	   b_kings_rook_moved  : false;
	   b_queens_rook_moved : false );

function is_black(c : char) : boolean;
begin
  is_black := (c <> ' ') and (c < 'b');
end;

function is_white(c : char) : boolean;
begin
  is_white := c > 'R';
end;

function square_colour(sq : char) : colour;
begin
  if (sq > 'r') then
    square_colour := white
  else
    square_colour := black;
end;

function valid_pawn(var g : game) : boolean;
var
  col : byte;
begin
  with g.move^ do
    begin
      if (from_col <> to_col) then
	valid_pawn := false;
      col := from_col;
      if (c = white) then
	begin
	  valid_pawn :=
	    (from_row < 8) and (b[to_row][col] = ' ') and
	    ((to_row = from_row + 1) or
	     ((from_row = 2) and (to_row = from_row + 2) and
	      (b[from_row + 1][col] = ' ')));
	end
      else
	begin
	  valid_pawn :=
	    (from_row > 1) and (b[to_row][to_col] = ' ') and
	    ((to_row = from_row - 1) or
	     ((from_row = 7) and (to_row = from_row - 2) and
	      (b[from_row - 1][from_col] = ' ')));
	end;
    end;
end;

function pawn_capture(var g : game) : boolean;
begin
  with g.move^ do
    begin
      if (to_col <> from_col + 1) and (to_col <> from_col - 1) then
	pawn_capture := false;
      if (c = white) then
	begin
	  if (to_row <> from_row + 1) then
	    pawn_capture := false;
	  if (is_black(b[to_row][to_col])) then
	    pawn_capture := true;
	  pawn_capture := (b[from_row][to_col] = 'P') and
			  (from_row = 5) and
			  (prev <> nil) and (prev^.piece = 'P') and
			  (prev^.from_row = 7) and
			  (prev^.to_row = 5) and
			  (prev^.to_col = to_col);
	end
      else
	begin
	  if (to_row <> from_row - 1) then
	    pawn_capture := false;
	  if (is_white(b[to_row][to_col])) then
	    pawn_capture := true;
	  pawn_capture := (b[from_row][to_col] = 'p') and
			  (from_row = 4) and
			  (prev <> nil) and (prev^.piece = 'p') and
			  (prev^.from_row = 2) and
			  (prev^.to_row = 4) and
			  (prev^.to_col = to_col);
	end;
    end;
end;

function valid_knight(var g : game) : boolean;
var
  dist_row : byte;
  dist_col : byte;
  square : char;
begin
  with g.move^ do
    begin
      dist_row := abs(from_row - to_row);
      dist_col := abs(from_col - to_col);
      if not ((dist_row = 2) and (dist_col = 1) or
	      (dist_row = 1) and (dist_col = 2)) then
	valid_knight := false;
      square := b[to_row][to_col];
      valid_knight := (square = ' ') or
		      (square_colour(square) <> c);
    end;
end;

function valid_bishop(var g : game) : boolean;
begin
{  with g.game^ do
    begin

    end; }
end;

const
  startX = 3;
  startY = 1;

procedure DWriteChar(x, y : byte; c : char); assembler;
asm
  mov   ax, $B800
  mov	es, ax
  mov	al, 80
  mov	cl, y
  dec	cl
  mul	cl
  mov   di, ax
  xor	ah, ah
  mov	al, x
  dec	al
  add	di, ax
  shl	di, 1
  mov	al, c
  mov	[es:di], al
end;

procedure DWriteAttr(x, y, attr : Byte); assembler;
asm
  mov	ax, $B800
  mov	es, ax
  mov	al, 80
  mov	cl, y
  dec	cl
  mul	cl
  mov   di, ax
  xor	ah, ah
  mov	al, x
  dec	al
  add	di, ax
  shl	di, 1
  mov	al, attr
  mov	[es:di+1], al
end;

procedure init_display_board(c : colour);
var
  i, j : word;
begin
  clrscr;
  i:= 0;
  gotoxy(1, startY);
  for i := 1 to 3 do
    begin
      writeln('                      ');
      writeln('                          ');
      writeln('                      ');
      writeln('                      ');
      writeln('                          ');
      writeln('                      ');
    end;
  writeln('                      ');
  writeln('                          ');
  writeln('                      ');
  writeln('                      ');
  writeln('                          ');
  writeln('  ____________________');
  if (c = white) then
    write(  '     a     b     c     d     e     f     g     h')
  else
    write(  '     h     g     f     e     d     c     b     a');
  for i := 0 to 3 do
    for j := 0 to 3 do
      begin
	DWriteAttr(startX + 9 + i * 12, startY + 1 + j * 6, byte(White) shl 4);
	DWriteAttr(startX + 3 + i * 12, startY + 4 + j * 6, byte(White) shl 4);
      end;
  for i := 1 to 8 do
    begin
      gotoxy(startX - 2, startY + i * 3 - 2);
      if (c = white) then
	write(9-i)
      else
	write(i);
    end;
end;

procedure display_board(var b : board; c : colour);
  var i, j : integer;
begin
  if (c = black) then
    for i := 0 to 7 do
      for j := 0 to 7 do
	DWriteChar(startX + 3 + j * 6, startY + 1 + i * 3, b[i+1][j+1])
  else
    for i := 0 to 7 do
      for j := 0 to 7 do
	DWriteChar(startX + 3 + j * 6, startY + 1 + i * 3, b[8-i][8-j]);
end;

procedure display_piece(row, col : byte; var b : board; c : colour);
begin
  if (c = black) then
    DWriteChar(startX - 3 + col * 6, startY - 2 + row * 3, b[row][col])
  else
    DWriteChar(startX + 51 - col * 6, startY + 25 - row * 3, b[row][col]);
end;

procedure display_board2(var b : board; c : colour);
var
  row, col : byte;
begin
  for row := 1 to 8 do
    for col := 1 to 8 do
      display_piece(row, col, b, c);
end;

var
  g : game;

begin
  g := initial_position;
  g.move := new(chess_move_p);
  g.move^ := initial_move;
  init_display_board(black);
  display_board2(b, black);
end.